---
title: "Tools to write and publish your code"
author: "Jean Feydy"
institute: ""
topic: ""
theme: "metropolis"
colortheme:
fonttheme:
fontsize: 11pt
linkcolor: black
urlcolor: blue
linkstyle: bold
aspectratio: 169
titlegraphic: 
logo:
date:
section-titles: false
toc: false
---


## Today's talk

**How to:**

- Collaborate with **git**.
- Create a personal **webpage**.
- Make your code **look clean**.
- Test your code **automatically**.

I will **focus on tools** which are:

- $\BR{Easy}$ to use, free of charge and with minimal overhead.
- **Accessible** to all of us -- i.e. no Inria- or Inserm-specific service.

`\begin{center}`{=latex}
$\BR{These slides are available on my webpage (Research tab, Talks section).}$
`\end{center}`{=latex}


# Collaborate with git


## Is your current workflow good enough?

Working in a **local folder** or in a Dropbox prevents you from:

- Keeping track of the **history** of your project.
- **Rolling back** modifications.
- Working in **parallel** with your $\BR{collaborators}$.
- Welcoming **direct feedback** from external users.


$\BR{Git}$ is the standard tool to **solve these issues**:

- **Free**, open source software.
- **Ubiquitous** both in academia and the industry.
- De-centralized, **offline** copies of the project history stay on your computer.
- $\BR{Synchronization}$ and authentication via a web platform: GitHub, GitLab...


## GitHub is the main platform worldwide, maintained by Microsoft

`\begin{center}`{=latex}
![](images/git/GitHub_1.png){height=95%}
`\end{center}`{=latex}


## Create a new shared folder, or *repository*

`\begin{center}`{=latex}
![](images/git/GitHub_2.png){height=95%}
`\end{center}`{=latex}


## Specify the main properties of your project

`\begin{center}`{=latex}
![](images/git/GitHub_3.png){height=95%}
`\end{center}`{=latex}


## A reference copy of the repository is now in the cloud

`\begin{center}`{=latex}
![](images/git/GitHub_4.png){height=95%}
`\end{center}`{=latex}


## Create a local copy (*clone*) using e.g. VSCode

`\begin{center}`{=latex}
![](images/git/Clone_1.png){height=95%}
`\end{center}`{=latex}

## Create a local copy (*clone*) using e.g. VSCode

`\begin{center}`{=latex}
![](images/git/Clone_2.png){height=95%}
`\end{center}`{=latex}

## Create a local copy (*clone*) using e.g. VSCode

`\begin{center}`{=latex}
![](images/git/Clone_3.png){height=95%}
`\end{center}`{=latex}

## Create a local copy (*clone*) using e.g. VSCode

`\begin{center}`{=latex}
![](images/git/Clone_4.png){height=95%}
`\end{center}`{=latex}

## Create a local copy (*clone*) using e.g. VSCode

`\begin{center}`{=latex}
![](images/git/Clone_5.png){height=95%}
`\end{center}`{=latex}



## A typical Git workflow

`\begin{center}`{=latex}
![](images/git/Current_directory.png){height=95%}
`\end{center}`{=latex}

## Interact with Git using GitKraken or a VSCode plugin

`\begin{center}`{=latex}
![](images/git/Git_extensions.png){height=95%}
`\end{center}`{=latex}

## My current Git repositories

`\begin{center}`{=latex}
![](images/git/Git_panel.png){height=95%}
`\end{center}`{=latex}


## The Git Lens view

`\begin{center}`{=latex}
![](images/git/Git_Lens.png){height=95%}
`\end{center}`{=latex}


## The Git Graph view -- basically the same thing!

`\begin{center}`{=latex}
![](images/git/Git_Graph.png){height=95%}
`\end{center}`{=latex}


## Step 1: edit some file

`\begin{center}`{=latex}
![](images/git/Edit_1.png){height=95%}
`\end{center}`{=latex}

## Step 2: switch to the Git panel

`\begin{center}`{=latex}
![](images/git/Edit_2.png){height=95%}
`\end{center}`{=latex}

## Step 3: check your modifications

`\begin{center}`{=latex}
![](images/git/Edit_3.png){height=95%}
`\end{center}`{=latex}

## Step 4: look at the graph history 

`\begin{center}`{=latex}
![](images/git/Edit_4.png){height=95%}
`\end{center}`{=latex}

## Step 5: give a name to your modification and *commit* 

`\begin{center}`{=latex}
![](images/git/Edit_6.png){height=95%}
`\end{center}`{=latex}

## Step 7: synchronize your local version with the GitHub cloud

`\begin{center}`{=latex}
![](images/git/Edit_7.png){height=95%}
`\end{center}`{=latex}

## Step 8: done!

`\begin{center}`{=latex}
![](images/git/Edit_8.png){height=95%}
`\end{center}`{=latex}

## Step 9: check online

`\begin{center}`{=latex}
![](images/git/Edit_9.png){height=95%}
`\end{center}`{=latex}

## Step 9: check online

`\begin{center}`{=latex}
![](images/git/Edit_10.png){height=95%}
`\end{center}`{=latex}



## Git allows you to collaborate *asynchronously*

`\begin{center}`{=latex}
![](images/git/SurvivalGPU_1.png){height=95%}
`\end{center}`{=latex}

## Development *branches* have names

`\begin{center}`{=latex}
![](images/git/SurvivalGPU_2.png){height=95%}
`\end{center}`{=latex}

## Right click on a commit to create a new branch

`\begin{center}`{=latex}
![](images/git/SurvivalGPU_3.png){height=95%}
`\end{center}`{=latex}

## Right click on a branch label to "check it out" or merge it

`\begin{center}`{=latex}
![](images/git/SurvivalGPU_4.png){height=95%}
`\end{center}`{=latex}


## Click on a commit to inspect changes

`\begin{center}`{=latex}
![](images/git/SurvivalGPU_5.png){height=95%}
`\end{center}`{=latex}

## Diffs and roll-backs are very useful to debug!

`\begin{center}`{=latex}
![](images/git/SurvivalGPU_6.png){height=95%}
`\end{center}`{=latex}


# Create a personal webpage

## Why should you care?

A personal webpage is:

- The standard "CV format" in mathematics and computer science.
- $\BR{Mandatory in academia}$ -- ResearchGate or LinkedIn are *not* enough.
- $\BR{Helpful}$ to land a data science job.

Create a webpage **as soon as possible**:

- **Present** your projects without being constrained by a journal template.
- Help people **find** your slides, code, supplementary materials...
- Get referenced by Google **before** you're actively looking for a job.


## Step 1: create a GitHub organization with the name of your future website

`\begin{center}`{=latex}
![](images/website/Organization_1.png){height=95%}
`\end{center}`{=latex}

## Step 1: create a GitHub organization that bears the name of your website

`\begin{center}`{=latex}
![](images/website/Organization_2.png){height=95%}
`\end{center}`{=latex}

## Step 1: create a GitHub organization that bears the name of your website

`\begin{center}`{=latex}
![](images/website/Organization_3.png){height=95%}
`\end{center}`{=latex}

## Step 1: create a GitHub organization that bears the name of your website

`\begin{center}`{=latex}
![](images/website/Organization_4.png){height=95%}
`\end{center}`{=latex}

## Step 2: create a repository called yoursite.github.io

`\begin{center}`{=latex}
![](images/website/Organization_5.png){height=95%}
`\end{center}`{=latex}

## Step 2: create a repository called yoursite.github.io

`\begin{center}`{=latex}
![](images/website/Organization_6.png){height=95%}
`\end{center}`{=latex}

## Step 3: create some minimal files

`\begin{center}`{=latex}
![](images/website/Jekyll_1.png){height=95%}
`\end{center}`{=latex}

## Step 4: choose your Jekyll theme

`\begin{center}`{=latex}
![](images/website/Jekyll_2.png){height=95%}
`\end{center}`{=latex}

## Step 5: create a nice banner using e.g. the Midjourney AI

`\begin{center}`{=latex}
![](images/website/Jekyll_3.png){height=95%}
`\end{center}`{=latex}

## Step 6: create your front page

`\begin{center}`{=latex}
![](images/website/Jekyll_4.png){height=95%}
`\end{center}`{=latex}

## Step 7: commit, synchronize... and it's live!

`\begin{center}`{=latex}
![](images/website/Jekyll_5.png){height=95%}
`\end{center}`{=latex}

## A slightly more complete example

`\begin{center}`{=latex}
![](images/website/Jekyll_6.png){height=95%}
`\end{center}`{=latex}

## A slightly more complete example

`\begin{center}`{=latex}
![](images/website/Jekyll_7.png){height=95%}
`\end{center}`{=latex}

## Going further with Quarto -- include Python and R notebooks, etc.

`\begin{center}`{=latex}
![](images/website/Going_further.png){height=95%}
`\end{center}`{=latex}


# Make your code look clean

## Standard *formatters* now beautify your code automagically

These tools:

- Choose spacings, line breaks and parentheses for you.
- **Do not impact the meaning** of your code.
- Smooth out discrepancies between $\BR{different collaborators}$.

Some good options:

- **black** for Python (`\texttt{pip install "black[jupyter]"}`{=latex})
- **styler** for R (`\texttt{install.packages("styler")}`{=latex})

```{=latex}
\end{frame}

\begin{frame}[fragile]{Write some ugly code}

 \

\begin{minted}[bgcolor=blue!4,mathescape,tabsize=4,firstnumber=1,stripnl=false,fontsize=\footnotesize]{python}
def f(a, **kwargs):
    b= a +1
    c = b**2+a * 3
    return c + 1

r = f(1, b = [x**2 for x in range(10)], d= [y**3 for y in range(5)], 
      e = [z**4 for z in range(3)])





\end{minted}

\end{frame}

\begin{frame}[fragile]{Run ``\texttt{black code.py}'' or ``\texttt{black .}'' in your code folder}

 \

\begin{minted}[bgcolor=blue!4,mathescape,tabsize=4,firstnumber=1,stripnl=false,fontsize=\footnotesize]{python}
def f(a, **kwargs):
    b = a + 1
    c = b**2 + a * 3
    return c + 1


r = f(
    1,
    b=[x**2 for x in range(10)],
    d=[y**3 for y in range(5)],
    e=[z**4 for z in range(3)],
)
\end{minted}
```

# Test your code automatically

## Automatic testing is a key part of software engineering

Modern tools:

- Let you write test functions with **minimal overhead**.
- Run a full test suite every time you push a commit to GitHub.
- Send you an **e-mail** if your modifications have $\BR{introduced a bug}$.

This is key:

- To quickly identify mistakes.
- To let you $\BR{trust your collaborators}$ on large projects.
- To let users $\BR{trust your team}$.
- To let the **future** you trust the **current** you...
- To raise an alarm if a **dependency update** (PyTorch...) breaks your code.


## Automatic testing is known as *continuous integration*

**testthat** is the standard package for R. \
 \
A personal selection for Python:

- $\BR{pytest}$ to discover test functions in the repository.
- **hypothesis** to generate challenging test cases.
- **beartype** to check function inputs.
- **jaxtyping** to check the shapes of NumPy arrays and Torch tensors.
- $\BR{codecov}$ to highlight the parts of the code that are yet to be tested.



## Step 1: create a file named `\texttt{test\_*.py}`{=latex} and a function `\texttt{test\_*(...)}`{=latex}

`\begin{center}`{=latex}
![](images/test/pytest_1.png){height=95%}
`\end{center}`{=latex}

## Step 2: find bugs

`\begin{center}`{=latex}
![](images/test/pytest_2.png){height=95%}
`\end{center}`{=latex}

## Step 3: fix them!

`\begin{center}`{=latex}
![](images/test/pytest_3.png){height=95%}
`\end{center}`{=latex}

## Step 4: setup automated tests for every GitHub push

`\begin{center}`{=latex}
![](images/test/pytest_4.png){height=95%}
`\end{center}`{=latex}

## Step 5: tests run for free on the GitHub servers

`\begin{center}`{=latex}
![](images/test/pytest_5.png){height=95%}
`\end{center}`{=latex}

## Success: your tests passed!

`\begin{center}`{=latex}
![](images/test/pytest_6.png){height=95%}
`\end{center}`{=latex}

## Failure: you will get an e-mail!

`\begin{center}`{=latex}
![](images/test/pytest_7.png){height=95%}
`\end{center}`{=latex}

## Failure: inspect the test run to identify the bug

`\begin{center}`{=latex}
![](images/test/pytest_8.png){height=95%}
`\end{center}`{=latex}


## Codecov: a nice web API to inspect your test coverage

`\begin{center}`{=latex}
![](images/test/codecov_1.png){height=95%}
`\end{center}`{=latex}

## Codecov: a nice web API to inspect your test coverage

`\begin{center}`{=latex}
![](images/test/codecov_2.png){height=95%}
`\end{center}`{=latex}


## Conclusion

The tools we discussed today let you:

- **Collaborate efficiently** with your team.
- Control the **flow of information** about your work.


I strongly advise you to $\BR{try them out}$:

- **Smooth** learning curve.
- Save **tons of time** (debugging, lost versions...).
- $\BR{Be credible}$ in team meetings, conferences, job interviews.



```{=latex}
\end{frame} 

\begin{frame}[standout]
Please feel free to ask me questions anytime!
```


